package com.example.wtf.controller;

import com.example.wtf.exception.ResourceNotFound;
import com.example.wtf.model.Question;
import com.example.wtf.repository.questionRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@RestController
@RequestMapping("/questions")
public class questionController {
    @Autowired
    private questionRepository repo;

    @GetMapping("/get")
    public @ResponseBody CollectionModel<EntityModel<Question>> getQuestions() {
        List<EntityModel<Question>> questions = new ArrayList<>();

        for (Question question : repo.findAll()) {
            EntityModel<Question> questionResource = EntityModel.of(
                    question,
                    linkTo(methodOn(questionController.class).getQuestion(question.getId())).withSelfRel(),
                    linkTo(methodOn(questionController.class).updateQuestion(question.getId(), null)).withRel("Update question"),
                    linkTo(methodOn(questionController.class).deleteQuestion(question.getId())).withRel("Delete question")
            );
            questions.add(questionResource);
        }

        return CollectionModel.of(
                questions,
                linkTo(methodOn(questionController.class).addQuestion(null)).withRel("Add Question")
        );
    }

    @GetMapping("/get/{id}")
    public @ResponseBody EntityModel<Question> getQuestion(@PathVariable Long id) {
        Question question = repo.findQuestionById(id);
        if (question == null) {
            throw new ResourceNotFound(HttpStatus.NOT_FOUND, "Question not found");
        }
        return EntityModel.of(
                question,
                linkTo(methodOn(questionController.class).updateQuestion(question.getId(), null)).withRel("Update question"),
                linkTo(methodOn(questionController.class).deleteQuestion(question.getId())).withRel("Delete question"),
                linkTo(methodOn(questionController.class).getQuestions()).withRel("Questions")
        );
    }

    @PostMapping("/add")
    public @ResponseBody EntityModel<Question> addQuestion(@RequestBody Question question) {
        repo.save(question);
        return EntityModel.of(
                question,
                linkTo(methodOn(questionController.class).getQuestion(question.getId())).withSelfRel(),
                linkTo(methodOn(questionController.class).updateQuestion(question.getId(), null)).withRel("Update question"),
                linkTo(methodOn(questionController.class).deleteQuestion(question.getId())).withRel("Delete question"),
                linkTo(methodOn(questionController.class).getQuestions()).withRel("Questions")
        );
    }

    @PutMapping("/update/{id}")
    public @ResponseBody EntityModel<Question> updateQuestion(@PathVariable Long id,
                                                              @RequestBody Question question) {
        Question q = repo.findQuestionById(id);
        if (q == null) throw new ResourceNotFound(HttpStatus.NOT_FOUND, "Question not found");
        if (question.getContent() != null && !question.getContent().isEmpty()) {
            q.setContent(question.getContent());
        }
        if (question.getAnswerA() != null && !question.getAnswerA().isEmpty()) {
            q.setAnswerA(question.getAnswerA());
        }
        if (question.getAnswerB() != null && !question.getAnswerB().isEmpty()) {
            q.setAnswerB(question.getAnswerB());
        }
        if (question.getAnswerC() != null && !question.getAnswerC().isEmpty()) {
            q.setAnswerC(question.getAnswerC());
        }
        if (question.getAnswerD() != null && !question.getAnswerD().isEmpty()) {
            q.setAnswerD(question.getAnswerD());
        }
        if (question.getcAnswer() != null && !question.getcAnswer().isEmpty()) {
            q.setcAnswer(question.getcAnswer());
        }
        repo.save(q);
        return EntityModel.of(
                q,
                linkTo(methodOn(questionController.class).getQuestion(id)).withSelfRel(),
                linkTo(methodOn(questionController.class).deleteQuestion(id)).withRel("Delete question"),
                linkTo(methodOn(questionController.class).getQuestions()).withRel("Questions")
        );
    }

    @DeleteMapping("delete/{id}")
    public @ResponseBody EntityModel<Question> deleteQuestion(@PathVariable Long id) {
        Question q = repo.findQuestionById(id);
        if (q == null) throw new ResourceNotFound(HttpStatus.NOT_FOUND, "Question not found");

        repo.delete(q);

        return EntityModel.of(
                q,
                linkTo(methodOn(questionController.class).getQuestions()).withRel("Questions")
        );
    }
}
